// SPDX-License-Identifier: GPL-2.0+
/*
 * based on board/freescale/imx8ulp-evk/imx8ulp-evk.c
 *
 * Copyright (C) 2024 Atmark Techno, Inc. All Rights Reserved.
 */

#include <common.h>

#include <asm/arch/clock.h>
#include <asm/arch/imx8ulp-pins.h>
#include <asm/arch/m33_proto.h>
#include <asm/arch/pcc.h>
#include <asm/arch/sys_proto.h>
#include <asm/gpio.h>
#include <dt-bindings/power/imx8ulp-power.h>
#include <fuse.h>
#include <i2c.h>
#include <miiphy.h>
#include <miiphy.h>
#include <mmc.h>
#include <netdev.h>
#include <netdev.h>
#include <power-domain.h>

#include "board_kotai.h"

DECLARE_GLOBAL_DATA_PTR;

void setup_m33_console(void)
{
	const char *m33_console = env_get("m33_console");
	char *s;
	u32 pin_tx, pin_rx, status;
	int rc;

	if (!m33_console || !strcmp(m33_console, "none"))
		return;

	pin_tx = simple_strtoul(m33_console, &s, 10);
	if (s[0] != '/')
		goto inval;
	pin_rx = simple_strtoul(s + 1, &s, 10);
	if (s[0] != 0 || pin_tx > 24 || pin_rx > 24)
		goto inval;

	rc = m33_send(M33_DEBUG_CONSOLE | (pin_tx << 8) | (pin_rx << 16));
	if (rc) {
		status = EIO;
		goto out;
	}

	rc = m33_recv(10, &status);
	if (rc)
		status = EIO;

out:
	if (status)
		printf("Setting m33 console failed for %u/%u: %u\n",
		       pin_tx, pin_rx, status);
	else
		printf("m33 console set to %u/%u\n", pin_tx, pin_rx);

	return;

inval:
	printf("m33_console must be 'none' or [tx]/[rx] (e.g. 18/19)\n");
}

#if IS_ENABLED(CONFIG_FEC_MXC)
#define ENET_CLK_PAD_CTRL	(PAD_CTL_PUS_UP | PAD_CTL_DSE | PAD_CTL_IBE_ENABLE)
static iomux_cfg_t const enet_clk_pads[] = {
	IMX8ULP_PAD_PTE19__ENET0_REFCLK | MUX_PAD_CTRL(ENET_CLK_PAD_CTRL),
};

static int setup_fec(void)
{
	/*
	 * Since ref clock and timestamp clock are from external,
	 * set the iomux prior the clock enablement
	 */
	imx8ulp_iomux_setup_multiple_pads(enet_clk_pads, ARRAY_SIZE(enet_clk_pads));

	/* Select enet time stamp clock: 001 - External Timestamp Clock */
	cgc1_enet_stamp_sel(1);

	/* enable FEC PCC */
	pcc_clock_enable(4, ENET_PCC4_SLOT, true);
	pcc_reset_peripheral(4, ENET_PCC4_SLOT, false);

	return 0;
}

int board_phy_config(struct phy_device *phydev)
{
	if (phydev->drv->config)
		phydev->drv->config(phydev);
	return 0;
}
#endif

int board_init(void)
{
#if defined(CONFIG_FEC_MXC)
	setup_fec();
#endif
	return 0;
}

int board_early_init_f(void)
{
	return 0;
}

static void set_serial(void)
{
	u32 product_id_val, lot_val;
	char serial[13 /* product (4), lot (8), \0 (1) */];
	char *old_serial;
	int ret;

	ret = fuse_read(EFUSE_KOTAI_BANK, EFUSE_KOTAI_WORD, &product_id_val);
	if (ret) {
		printf("Error: Can not read seihin kanri id\n");
		return;
	}

	ret = fuse_read(EFUSE_KOTAI_BANK, EFUSE_KOTAI_WORD + 1, &lot_val);
	if (ret) {
		printf("Error: Can not read lot number\n");
		return;
	}

	if (product_id_val == 0 && lot_val == 0) {
		printf("Error: serial number not written in fuses\n");
		return;
	}

	snprintf(serial, sizeof(serial), "%04X%08X", product_id_val, lot_val);

	old_serial = env_get("serial#");
	if (old_serial && old_serial[0] != '\0' && strcmp(serial, old_serial)) {
		/* board previously booted somewhere else, clear mac addresses */
		printf("serial# changed: Clearing MAC addresses in env\n");
		env_set("ethaddr", NULL);
		env_set("eth1addr", NULL);
	}

	env_set("serial#", serial);
}

int board_late_init(void)
{
#if CONFIG_IS_ENABLED(ENV_IS_IN_MMC)
	board_late_mmc_env_init();
#endif

	set_serial();
	setup_m33_console();
	return 0;
}

void board_quiesce_devices(void)
{
	/* Disable the power domains may used in u-boot before entering kernel */
#if CONFIG_IS_ENABLED(POWER_DOMAIN)
	struct udevice *scmi_devpd;
	int ret, i;
	struct power_domain pd;
	ulong ids[] = {
		IMX8ULP_PD_FLEXSPI2, IMX8ULP_PD_USB0, IMX8ULP_PD_USDHC0,
		IMX8ULP_PD_USDHC1, IMX8ULP_PD_USDHC2_USB1, IMX8ULP_PD_DCNANO,
		IMX8ULP_PD_MIPI_DSI};

	ret = uclass_get_device(UCLASS_POWER_DOMAIN, 0, &scmi_devpd);
	if (ret) {
		printf("Cannot get scmi devpd: err=%d\n", ret);
		return;
	}

	pd.dev = scmi_devpd;

	for (i = 0; i < ARRAY_SIZE(ids); i++) {
		pd.id = ids[i];
		ret = power_domain_off(&pd);
		if (ret)
			printf("power_domain_off %lu failed: err=%d\n", ids[i], ret);
	}
#endif
}
